@@ -74,6 +74,7 @@ struct Reg_Remote {
 	char str_RestrictIP[SIZE_OF_RESTRICTIP];
 	int dw_Password;
 	char str_Password[SIZE_OF_PASSWORD];
+	char str_OldPassword[SIZE_OF_PASSWORD];
 };
 
 struct Reg_Objective {
@@ -107,11 +108,14 @@ int Objective_Display(char *source, char *dest, int size);
 int Restart(char *source, char *dest, int size, int one, int two);
 int Network_Set(char *source, char *dest, int size);
 int Objective_Result(char *, char *, int);
+int Write_Objective_To_File(struct Reg_Objective, char *, int, int);
+int Modify_Objective_In_File(struct Reg_Objective, int, char *, int);
 
 FILE *Find_First_Objective();
 FILE *Find_First_Network();
 int Get_Next_Objective(FILE * configfile, struct Reg_Objective *objective);
 int Get_Next_Network(FILE * configfile, struct Reg_Host *host_struct);
+int Write_Network_To_File (struct Reg_Config config_struct, struct Reg_Host *RHead, char *dest, int size);
 int Close_Objective_File(FILE * configfile);
 
 int ReadObjectives();
@@ -131,11 +135,13 @@ char *GetNextArgument(char *source, char *destvar, int varlength,
 void *Load_Config_File();
 int Read_Config_From_File(struct Reg_Config *);
 void Clear_Config_File(void *location);
+int Clear_Objectives_From_File(char *dest, int size);
 int Grab_RAMConfig_Line(char *source, char *dest, int size);
 
 int getnetwork(char *string, char *host, int length, int *protocol);
 
 int Read_Remote_From_File(struct Reg_Remote *remote_struct);
+int Write_Remote_To_File (struct Reg_Remote remote_struct, char *dest, int size);
 
 int iswebfilename(char *string);
 int isstdout(char *string);
