@@ -1,12 +1,12 @@
 /**********************************************************
  *
- * Snare for Solaris version 2.5.6
+ * Snare for Solaris version 2.5.7
  *
  * Author: InterSect Alliance Pty Ltd
  *
  * Copyright 2001-2006 InterSect Alliance Pty Ltd
  *
- * Last Modified: 14/06/2006
+ * Last Modified: 10/01/2007
  *
  * Available under the terms of the GNU Public Licence.
  * - See www.gnu.org
@@ -140,9 +140,6 @@
 #define MY_AUT_TRAILER_MAGIC BSWAP_16(AUT_TRAILER_MAGIC)
 #endif
 
-// Comment this out to ensure logs are not written out to a file.
-//#define DEBUGLOGFILE 1
-//#define DEBUG 1 //Extra debugging
 void DebugMsg(const char *pszFormat, ...);
 
 // Linked List Functions
@@ -197,7 +194,6 @@ union semun {
 // Define an error variable so we can communicate with auditsvc
 extern int errno;
 int caught_pipe;                // variable that lets us know when a SIGPIPE has been received
-int caught_kill;                // variable that lets us know when a SIGKILL has been received
 int caught_usr1;                // variable that lets us know when a SIGUSR1 has been received
 
 uint_t CheckTrailer(char inbyte, int reset);
@@ -265,8 +261,8 @@ int main(int argc, char *argv[])
 	int auditstatus;
 	long auditpolicy=0;
 	int pid=0,pid2=0,pid3=0,pid4=0,ppid=0;	// process ids
-	//int webpid = 0;
-	pthread_t thread[1]; 
+	int webpid = 0;
+	pthread_t thread; 
 
 	// NODE that matches the objective search term.
 	Node *nodematch;
@@ -444,7 +440,11 @@ int main(int argc, char *argv[])
 	
 		
 		if (remote_allow) {
-			pthread_create(&thread[0], NULL, (void *)start_web, NULL);
+			if (!pthread_create(&thread, NULL, (void *)start_web, NULL)) {
+				//DebugMsg("Web thread created successfully: %d",thread);
+			} else {
+				DebugMsg("ERROR: Web thread failed");
+			}
 		}
 		
 		
@@ -554,10 +554,13 @@ int main(int argc, char *argv[])
 								if(!auditdatacheck) {
 #ifdef DEBUG
 									adc_count++;
-									if (adc_count > 100 && adc_count%10 == 0) DebugMsg("No data for a while: %d", adc_count);
+									if (adc_count > 100 && adc_count%50 == 0) DebugMsg("No data for a while: %d", adc_count);
 #endif //DEBUG
 									continue;
 #ifdef DEBUG
+								} else if (auditdatacheck < 0) {
+									DebugMsg("Audit data check failed, breaking loop reset buffers");
+									break;
 								} else {
 									adc_count = 0;
 #endif //DEBUG
@@ -607,6 +610,13 @@ int main(int argc, char *argv[])
 									}
 								}
 							}
+							//DebugMsg("Audit failed, falling back to parent thread");
+							//if(argc == 1) { audit_off(); }
+							//kill(pid2,SIGTERM); waitpid(pid2,NULL,NULL);
+							//kill(pid3,SIGTERM); waitpid(pid3,NULL,NULL);
+							//kill(pid4,SIGTERM); waitpid(pid4,NULL,NULL);
+							//kill(ppid,SIGPIPE);
+							exit(0);
 							break;
 						}
 
@@ -644,7 +654,8 @@ int main(int argc, char *argv[])
 											bytesleft = RecordFromRingbuffer(writebuffer, 
 													&recordnumber);
 											if( bytesleft <= 0 ) { // error ocurred
-												kill(ppid,SIGTERM); waitpid(pid,NULL,NULL);
+												//kill this thread and let the parent clean up and restart
+												//kill(ppid,SIGTERM); waitpid(pid,NULL,NULL);
 												if(argc == 1) { audit_off(); }
 												exit(1);
 											}
@@ -685,7 +696,7 @@ int main(int argc, char *argv[])
 										ssize_t readbytes;
 										ssize_t writebytes;
 										char buf[4096];
-										pid_t ppid=getppid();
+										pid_t ppid2=getppid();
 
 										for(i=1;i<argc;i++) {
 											DebugMsg("Processing %s..\n",argv[i]);
@@ -699,7 +710,7 @@ int main(int argc, char *argv[])
 											close(auditfile);
 										}
 
-										if (ppid) {
+										if (ppid2) {
 											kill(ppid,SIGUSR1);
 										}
 
@@ -725,23 +736,26 @@ int main(int argc, char *argv[])
 											// Just in case.
 											if (argc == 1) { audit_off(); }
 											unlink("/var/run/snarecore.pid");
-											kill(ppid,SIGTERM); // waitpid(pid,NULL,NULL);
 											kill(pid2,SIGTERM); waitpid(pid2,NULL,NULL);
 											kill(pid3,SIGTERM); waitpid(pid3,NULL,NULL);
 											kill(pid4,SIGTERM); waitpid(pid4,NULL,NULL);
+											kill(ppid,SIGPIPE);
+											if (remote_allow) {
+												DebugMsg("Terminating web server...");
+											}
 											exit(1);
 										} else if (error == EBUSY) {
 											// Auditing is already being performed by another process
 											DebugMsg("Another process has control of audit.\n  Please check that there is not another Snare process already running, and verify that the standard Solaris audit daemon is not still active.");
 											if (argc == 1) { audit_off(); }
-											if (thread[0]) {
-												kill(thread[0],SIGTERM);
+											if (thread) {
+												kill(thread,SIGTERM);
 											}
 
 											kill(ppid,SIGTERM); waitpid(ppid,NULL,NULL);
 											exit(1);
 										} else if (error == EINTR) {
-											DebugMsg("Interrupt received.\n");
+											DebugMsg("Interrupt received.");
 											// Signal handler has been tripped.
 									   		// This is likely to mean that the
 									   		// praudit program has died.
@@ -750,9 +764,16 @@ int main(int argc, char *argv[])
 											// kill(pid2,SIGTERM); waitpid(pid2,NULL,NULL);
 											// kill(ppid,SIGTERM); // waitpid(pid,NULL,NULL);
 											if(argc == 1) { audit_off(); }
+											DebugMsg("killing pid2...");
 											kill(pid2,SIGTERM); waitpid(pid2,NULL,NULL);
+											DebugMsg("killing pid3...");
 											kill(pid3,SIGTERM); waitpid(pid3,NULL,NULL);
+											DebugMsg("killing pid4...");
 											kill(pid4,SIGTERM); waitpid(pid4,NULL,NULL);
+											kill(ppid,SIGPIPE);
+											if (remote_allow) {
+												DebugMsg("Terminating web server...");
+											}
 											exit(0);
 										} else if (error == EBADF) {
 											DebugMsg("Pipe has been closed.\n");
@@ -764,6 +785,7 @@ int main(int argc, char *argv[])
 											kill(pid2,SIGTERM); waitpid(pid2,NULL,NULL);
 											kill(pid3,SIGTERM); waitpid(pid3,NULL,NULL);
 											kill(pid4,SIGTERM); waitpid(pid4,NULL,NULL);
+											kill(ppid,SIGPIPE);
 											exit(1);
 										} else {
 											sleep(1);	// Limit respawns for no reason
@@ -805,7 +827,7 @@ int main(int argc, char *argv[])
 			}
 
 			logbuffercount=0;
-			while (continueloop && !caught_kill) {
+			while (continueloop && !caught_kill && !caught_pipe) {
 				if (spaceavail==0) {
 					// Oh dear.. filled our buffer without seeing a newline. Must be filled with corrupt data from praudit.
 					// Clear it.
@@ -832,6 +854,12 @@ int main(int argc, char *argv[])
 					} else {
 						continue;
 					}
+				} else if (datacheck < 0) {
+#ifdef DEBUG
+					DebugMsg("Select error, sleeping...");
+#endif //DEBUG
+					sleep(1);
+					continue;
 				}
 
 				bytesread=read(praudit2daemon[0],bufferptr,spaceavail);
@@ -839,7 +867,10 @@ int main(int argc, char *argv[])
 					break;
 				}
 
-				if (bytesread < 1) continue;
+				if (bytesread < 1) {
+					DebugMsg("MAIN: waiting 1...");
+					continue;
+				}
 
 				tbuffer=bufferptr+bytesread-1;
 				tbuffer++;*tbuffer=0;
@@ -944,8 +975,11 @@ int main(int argc, char *argv[])
 				close(daemon2praudit[1]);
 		
 				continueloop=0;
-				if (thread[0]) {
-					kill(thread[0],SIGTERM);
+				if (thread) {
+#ifdef DEBUG
+					DebugMsg("killing web server...");
+#endif //DEBUG
+					pthread_join(thread,NULL);
 				}
 			
 				DestroyList();
@@ -1411,7 +1445,7 @@ int open_audit_network(char *string)
 	int port = 6161, protocol=SOCKETTYPE_UDP;	// Default port, protocol
 	char *pos, *pos2;
 	struct hostent *hp;
-	pthread_t thread[1]; 
+	pthread_t thread; 
 
 	// Initialise desthost.
 	strncpy(desthost, "localhost", MAX_HOSTID);
@@ -3082,14 +3116,27 @@ void DebugMsg(const char *pszFormat, ...)
 
 void start_web () {
 	int ret=0;
+	time_t lasttime=(time_t)0;
 	// Child process - Run our web server
+	//DebugMsg("web init %d", pthread_self());
 	while(InitWebServer(remote_webport,remote_restrictip,remote_password) == -1) {
 		// Problem binding. Sleep for a moment.
 		sleep(5);
 	}
-	while((ret = StartThread()) == 1) {
-		// Problem binding. Keep trying.
-		sleep(5);
+	//DebugMsg("web start");
+
+	while (!caught_kill && !caught_pipe) {
+		ret = ListenThread(&lasttime);
+
+		if (!ret) break;
 	}
-	return;
+	CloseWebServer();
+	//while((ret = StartThread()) == 1) {
+		// Problem binding. Keep trying.
+		//sleep(5);
+	//}
+#ifdef DEBUG
+	DebugMsg("Web server is RETURNING");
+#endif //DEBUG
+	pthread_exit(NULL);
 }
