@@ -26,21 +26,25 @@
 int http_listen_socket;
 int http_message_socket;
 
-// Signals
-//int caught_pipe = 0;		/* variable that lets us know when a SIGPIPE has been received */
-int caught_kill = 0;		/* variable that lets us know when a SIGKILL has been received */
-
-
 // NOTE: This password sits in RAM.
 // A user with appropriate access could pull it out.
 char SnarePass[256];
 char SnareIP[MAX_RESTRICTIP][17];
 char SnareHost[MAX_RESTRICTIP][SIZE_OF_CLIENTNAME];
 int SnareIP_count=0;
+int caught_web_kill = 0;
 extern int EvtCount;
 
 void webkill(sig) {
-	caught_kill++;
+#ifdef DEBUG
+	DebugMsg("WEBKILL called");
+#endif //DEBUG
+	caught_web_kill++;
+	if(http_listen_socket > 0) {
+		shutdown(http_listen_socket,SHUT_RDWR);
+		close(http_listen_socket);
+		http_listen_socket=-1;
+	}
 	return;
 }
 
@@ -57,8 +61,8 @@ int InitWebServer(unsigned short port, char *ip, char *password)
 	http_listen_socket = NULL;
 	http_message_socket = NULL;
 
-	signal(SIGPIPE,SIG_IGN);
-	// signal(SIGTERM,webkill);
+	signal(SIGPIPE,webkill);
+	signal(SIGTERM,webkill);
 
 	if (password) {
 		strncpy(SnarePass, password, 256);
@@ -113,6 +117,7 @@ int InitWebServer(unsigned short port, char *ip, char *password)
 
 int CloseWebServer()
 {
+	DebugMsg("Closing web server");
 	if(http_message_socket > 0) {
 		shutdown(http_message_socket,SHUT_RDWR);
 		close(http_message_socket);
@@ -132,11 +137,21 @@ int HandleConnect()
 	int retval;
 	// 8k buffer for input and output. Suggest we may need more for output.
 	char HTTPBuffer[MAX_HTTPBUFFER];
+	char HTTPBufferTemp[MAX_HTTPBUFFER];
 	char HTTPOutputBuffer[MAX_HTTPBUFFER];
 	char *pos;
 	int size = 0;
 	int header = 1;
 
+	struct timeval tv;
+	fd_set webread;
+
+	tv.tv_sec = 1;
+	tv.tv_usec = 0;
+	FD_ZERO(&webread);
+	FD_SET(http_message_socket,&webread);
+
+
 	retval =
 	    recv(http_message_socket, HTTPBuffer, sizeof(HTTPBuffer), 0);
 	// NOTE: Should probably do something about requests that are bigger than
@@ -154,6 +169,20 @@ int HandleConnect()
 
 	HTTPBuffer[retval] = '\0';
 
+	if (!strstr(HTTPBuffer,"\r\n\r\n")) {
+		int tempval;
+		//fopen patch, sleep then check if there is anything left to grab
+		sleep(2);
+		if (select(0,&webread,NULL,NULL,&tv) > 0) {
+			tempval = recv(http_message_socket,HTTPBufferTemp,sizeof (HTTPBufferTemp),0 );
+			HTTPBufferTemp[tempval]='\0';
+			strncat(HTTPBuffer,HTTPBufferTemp,sizeof(HTTPBuffer));
+		} else {
+			close(http_message_socket);
+			return(1);
+		}
+	}
+
 	// Hunt down the authentication string
 	// Dont ask for authentication if the password is blank.
 	pos = strstr(HTTPBuffer, "Authorization: Basic ");
@@ -235,7 +264,7 @@ int StartThread()
 	int ret = 0;
 	time_t lasttime=(time_t)0;
 
-	while (!caught_kill) {
+	while (!caught_web_kill) {
 		ret = ListenThread(&lasttime);
 
 		if (!ret)
@@ -243,7 +272,7 @@ int StartThread()
 	}
 	CloseWebServer();
 
-	if (caught_kill) {
+	if (caught_web_kill) {
 		return (-1);
 	}
 
@@ -254,6 +283,7 @@ int StartThread()
 	return (0);
 }
 
+
 int ListenThread(time_t *lasttime)
 {
 	struct sockaddr_in from;
@@ -269,14 +299,15 @@ int ListenThread(time_t *lasttime)
 	fromlen = sizeof (from);
 
 	while (invalidrequest) {
-
 		http_message_socket =
 		    accept(http_listen_socket, (struct sockaddr *) &from,
 			   &fromlen);
 
 		//setsockopt(http_message_socket,SOL_SOCKET,SO_SNDTIMEO,&timeout,sizeof(timeout));
 
-		if(caught_kill) {
+		if(caught_web_kill) {
+			caught_web_kill = 0;
+			DebugMsg("WEB: caught kill signal, exitting...");
 			return(0);
 		}
 
